import * as React from 'react';
import { TextControl } from '@tesseract-design/web-base';
import clsx from 'clsx';
import { useFallbackId } from '@modal-sh/react-utils';

/**
 * Derived HTML element of the {@link EmailInput} component.
 */
export type EmailInputDerivedElement = HTMLInputElement;

/**
 * Props of the {@link EmailInput} component.
 */
export interface EmailInputProps extends Omit<React.HTMLProps<EmailInputDerivedElement>, 'size' | 'type' | 'label' | 'inputMode' | 'pattern' | 'autoComplete'> {
  /**
   * Short textual description indicating the nature of the component's value.
   */
  label?: React.ReactNode,
  /**
   * Short textual description as guidelines for valid input values.
   */
  hint?: React.ReactNode,
  /**
   * Size of the component.
   */
  size?: TextControl.Size,
  /**
   * Should the component display a border?
   */
  border?: boolean,
  /**
   * Should the component occupy the whole width of its parent?
   */
  block?: boolean,
  /**
   * Style of the component.
   */
  variant?: TextControl.Variant,
  /**
   * Is the label hidden?
   */
  hiddenLabel?: boolean,
  /**
   * Allowed domains for emails.
   */
  domains?: string[],
  /**
   * Visual length of the input.
   */
  length?: number,
  /**
   * Should the component display a list of suggested values?
   */
  autoComplete?: boolean,
}

/**
 * Component for inputting email addresses.
 */
export const EmailInput = React.forwardRef<EmailInputDerivedElement, EmailInputProps>(
  (
    {
      label,
      hint,
      size = 'medium' as const,
      border = false as const,
      block = false as const,
      variant = 'default' as const,
      hiddenLabel = false as const,
      className,
      id: idProp,
      style,
      domains = [],
      length,
      autoComplete = false as const,
      ...etcProps
    }: EmailInputProps,
    forwardedRef,
  ) => {
    const labelId = React.useId();
    const id = useFallbackId(idProp);

    const pattern = (
      Array.isArray(domains) && domains.length > 0
        ? `.+?@(${domains.join('|').replace(/\./g, '\\.')})$`
        : undefined
    );

    return (
      <div
        className={clsx(
          'relative rounded ring-secondary/50 group has-[:disabled]:opacity-50',
          'focus-within:ring-4',
          {
            'block': block,
            'inline-block align-middle': !block,
          },
          className,
        )}
        style={style}
      >
        {label && (
          <>
            <label
              data-testid="label"
              id={labelId}
              htmlFor={id}
              className={clsx(
                'absolute z-[1] w-full top-0.5 left-0 pointer-events-none pl-1 text-xxs font-bold group-focus-within:text-secondary text-primary leading-none bg-negative select-none',
                {
                  'sr-only': hiddenLabel,
                },
                {
                  'pr-10': size === 'small',
                  'pr-12': size === 'medium',
                  'pr-16': size === 'large',
                },
              )}
            >
              <span className="block w-full whitespace-nowrap h-[1.1em] overflow-hidden text-ellipsis">
                {label}
              </span>
            </label>
            {' '}
          </>
        )}
        <input
          {...etcProps}
          size={length}
          ref={forwardedRef}
          id={id}
          aria-labelledby={labelId}
          type="email"
          data-testid="input"
          autoComplete={autoComplete ? 'email' : undefined}
          pattern={pattern}
          className={clsx(
            'bg-negative rounded-inherit w-full peer block font-inherit tabular-nums',
            'focus:outline-0',
            'disabled:cursor-not-allowed',
            {
              'pl-4': variant === 'default',
              'pl-1.5 pt-4': variant === 'alternate',
            },
            {
              'pr-10 h-10 text-xxs': size === 'small',
              'pr-12 h-12 text-xs': size === 'medium',
              'pr-16 h-16': size === 'large',
            },
          )}
        />
        {hint && (
          <div
            data-testid="hint"
            className={clsx(
              'absolute left-0 px-1 pointer-events-none text-xxs leading-none w-full bg-negative select-none',
              {
                'bottom-0 pl-4 pb-1': variant === 'default',
                'top-0.5': variant === 'alternate',
              },
              {
                'pt-2': variant === 'alternate' && size === 'small',
                'pt-3': variant === 'alternate' && size !== 'small',
              },
              {
                'pr-10': size === 'small',
                'pr-12': size === 'medium',
                'pr-16': size === 'large',
              },
            )}
          >
            <div
              className="opacity-50 whitespace-nowrap w-full h-[1.1em] overflow-hidden text-ellipsis"
            >
              {hint}
            </div>
          </div>
        )}
        <div
          data-testid="indicator"
          className={clsx(
            'text-center flex items-center justify-center aspect-square absolute bottom-0 right-0 pointer-events-none select-none text-primary group-focus-within:text-secondary',
            {
              'w-10': size === 'small',
              'w-12': size === 'medium',
              'w-16': size === 'large',
            },
          )}
        >
          <svg
            className="w-6 h-6 fill-none stroke-current stroke-2 linejoin-round linecap-round"
            viewBox="0 0 24 24"
            role="presentation"
          >
            <path d="M4 4h16c1.1 0 2 .9 2 2v12c0 1.1-.9 2-2 2H4c-1.1 0-2-.9-2-2V6c0-1.1.9-2 2-2z" />
            <polyline points="22,6 12,13 2,6" />
          </svg>
        </div>
        {border && (
          <span
            data-testid="border"
            className="absolute z-[1] inset-0 rounded-inherit border-2 border-primary pointer-events-none peer-focus:border-secondary"
          />
        )}
      </div>
    );
  },
);

EmailInput.displayName = 'EmailInput';

EmailInput.defaultProps = {
  label: undefined,
  hint: undefined,
  size: 'medium' as const,
  border: false as const,
  block: false as const,
  variant: 'default' as const,
  hiddenLabel: false as const,
  domains: [],
  length: undefined,
  autoComplete: false as const,
};
